@@ -4,6 +4,7 @@ import os
 from dynaconf import Dynaconf
 
 SETTINGS_FILES = ["settings.toml", ".secrets.toml"]
+REPO_ID_PATH = os.path.join(os.path.abspath(os.path.curdir), ".embedops", "repo_id.yml")
 
 # get an absolute path to the director that this file is in
 abs_current_dir = os.path.dirname(os.path.abspath(__file__))
@@ -31,3 +32,26 @@ settings.configure(
         "embedops_cli.ci_config_loader",
     ]
 )
+
+
+def get_repo_id(path=REPO_ID_PATH):
+    """get repo_id from .embedops/repo_id.yml
+
+    Args:
+        path (string, optional):    Absolute file system path to .embedops/repo_id.yml.
+                                    Defaults to REPO_ID_PATH.
+
+    Returns:
+        str: the value stored in <repo_root>/.embedops/repo_id.yml:REPO_ID or None
+    """
+    dot_eo = Dynaconf(
+        load_dotenv=False,
+        settings_files=[path],
+        silent_errors=True,
+    )
+    dot_eo.configure(
+        LOADERS_FOR_DYNACONF=[
+            "dynaconf.loaders.yaml_loader",
+        ]
+    )
+    return dot_eo.get("repo_id")
