@@ -2,24 +2,24 @@
 `embedops_authorization`
 =======================================================================
 Managing the user login functions
-* Author(s): Bailey Steinfadt
 """
 import os
 import base64
 import pathlib
 import webbrowser
-from time import sleep
+from time import sleep, monotonic
 import logging
 import subprocess
 from datetime import datetime
 from operator import itemgetter
 from urllib.parse import urlparse
 import platform
+from dynaconf.vendor.toml.decoder import TomlDecodeError
+from dynaconf import Dynaconf
+from dynaconf.loaders import toml_loader
 import requests
 import dotenv
 import click
-from dynaconf import Dynaconf
-from dynaconf.loaders import toml_loader
 import boto3
 from embedops_cli.api.configuration import Configuration
 from embedops_cli.api.rest import ApiException
@@ -28,10 +28,15 @@ from embedops_cli.eo_types import (
     DockerRegistryException,
     LoginFailureException,
     UnauthorizedUserException,
+    NoUserAssignmentException,
     UnknownDockerException,
+    BadTomlFileException,
+    UserDeclinedException,
+    LoginTimeoutException,
 )
-from embedops_cli.api.default_api import DefaultApi
+from embedops_cli.api.api.default_api import DefaultApi
 from embedops_cli.config import settings
+from embedops_cli.utilities import echo_error_and_fix
 
 _logger = logging.getLogger(__name__)
 
@@ -47,6 +52,9 @@ def get_auth_token(secrets_file=user_secrets) -> str:
     secrets_file_settings = Dynaconf(SETTINGS_FILES=[secrets_file])
     try:
         key = secrets_file_settings.EMBEDOPS_AUTH_TOKEN
+    except TomlDecodeError:
+        # if the eosecrets gets corrupted we can get a parsing error
+        echo_error_and_fix(BadTomlFileException())
     except AttributeError:
         # eat the attribute error because it's expected when the user has not logged in
         pass
@@ -57,7 +65,7 @@ def set_auth_token(auth_token: str, secrets_file=user_secrets):
     """Set the Auth0 token in user secrets"""
     try:
         toml_loader.write(secrets_file, {"EMBEDOPS_AUTH_TOKEN": auth_token}, merge=True)
-    except (IOError) as exc:
+    except IOError as exc:
         raise LoginFailureException from exc
 
 
@@ -69,17 +77,19 @@ def set_docker_expiration_time(expires_at, secrets_file=user_secrets):
             {"EMBEDOPS_ECR_TOKEN_EXPIRES_AT": expires_at.timestamp()},
             merge=True,
         )
-    except (IOError) as exc:
+    except IOError as exc:
         raise LoginFailureException from exc
 
 
 def fetch_registry_token() -> str:
     """Retrieve a GitLab token for the user's group and store it"""
+    _logger.debug("Fetch registery token")
+
     user_client = get_user_client()
 
     user = user_client.get_my_user()
     if len(user.groups) < 1:
-        raise UnauthorizedUserException
+        raise NoUserAssignmentException
 
     first_group_membership = user.groups[0]
     org_id = first_group_membership.group.org_id
@@ -101,8 +111,11 @@ def get_docker_expiration_time(secrets_file=user_secrets):
     secrets_file_settings = Dynaconf(SETTINGS_FILES=[secrets_file])
     try:
         registry_token_expires_at = secrets_file_settings.EMBEDOPS_ECR_TOKEN_EXPIRES_AT
-    except AttributeError as exc:
-        raise UnauthorizedUserException from exc
+    except TomlDecodeError:
+        # if the eosecrets gets corrupted we can get a parsing error
+        echo_error_and_fix(BadTomlFileException())
+    except AttributeError:
+        pass  # If we don't have an expiration time it might be our first time logging in
 
     return registry_token_expires_at
 
@@ -115,6 +128,9 @@ def get_registry_token(secrets_file=user_secrets):
     try:
         registry_token_id = secrets_file_settings.EMBEDOPS_ECR_TOKEN_ID
         registry_token_secret = secrets_file_settings.EMBEDOPS_ECR_TOKEN_SECRET
+    except TomlDecodeError:
+        # if the eosecrets gets corrupted we can get a parsing error
+        echo_error_and_fix(BadTomlFileException())
     except AttributeError as exc:
         raise UnauthorizedUserException from exc
     if not registry_token_id or not registry_token_secret:
@@ -142,7 +158,7 @@ def set_registry_token(
             merge=True,
         )
 
-    except (IOError) as exc:
+    except IOError as exc:
         raise LoginFailureException from exc
 
 
@@ -190,7 +206,9 @@ def request_user_code(client_id, base_url):
 
     url = f"{base_url}/device/code"
 
-    code_req_response = requests.post(url, data=data)
+    code_req_response = requests.post(
+        url, data=data, timeout=int(settings.get("http_timeout", 60))
+    )
     return code_req_response.json()
 
 
@@ -207,30 +225,63 @@ def show_login_instructions(user_code):
     click.secho("-" * 80, fg="magenta")
 
 
-def launch_login_url(verification_url, base_url, device_code, client_id):
+def launch_login_url(
+    verification_url, base_url, device_code, client_id, timeout_max=60
+):
     """Launch the url for the user to log into EmbedOps"""
-    webbrowser.open_new(verification_url)
+    if os.environ.get("GITPOD_INSTANCE_ID") is None:
+        webbrowser.open_new(verification_url)
+    else:
+        click.secho(f"\nGitpod Users:")
+        click.secho("- check your pop-up blocker for login window")
+        click.secho(f"- OR open in browser at: {verification_url}\n")
+        sleep(2)
+        with subprocess.Popen(
+            ["gp", "preview", "--external", verification_url],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        ) as process:
+            process.communicate()
     click.secho("\nWaiting for verification in browser:")
-    click.secho("(To stop, press cancel in browser and type CTRL+C here)\n")
+    click.secho("(To stop, press cancel in browser or type CTRL+C here)\n")
+
+    time_start = monotonic()
     verified = False
+    url = f"{base_url}/token"
+    token_data = {
+        "device_code": device_code,
+        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
+        "client_id": client_id,
+    }
+
     while not verified:
-        url = f"{base_url}/token"
-        token_data = {
-            "device_code": device_code,
-            "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
-            "client_id": client_id,
-        }
         token_resp = None
         token_record = None
         try:
-            token_resp = requests.post(url, data=token_data)
+            token_resp = requests.post(
+                url, data=token_data, timeout=int(settings.get("http_timeout", 60))
+            )
         except RuntimeError as exc:
             raise LoginFailureException from exc
 
+        # Check if code is between 200 and 400
         if token_resp:
             token_record = token_resp.json()
             click.secho("\n")
             break
+
+        # Check if response says user denied
+        if (
+            token_resp.status_code == 403
+            and not "authorization_pending" in token_resp.text
+        ):
+            echo_error_and_fix(UserDeclinedException())
+
+        # Check if we've timed out
+        now = monotonic()
+        if (now - time_start) > timeout_max:
+            echo_error_and_fix(LoginTimeoutException())
+
         click.secho(".", nl=False, fg="magenta")
         sleep(0.1)
 
@@ -314,6 +365,8 @@ def docker_cli_login(aws_access_key_id, aws_secret_access_key):
 
 def login_to_registry(secrets_file=user_secrets):
     """Log into a docker registry"""
+    _logger.debug("Login to registry")
+
     try:
         registry_token_data = get_registry_token(secrets_file)
     except UnauthorizedUserException as exc:
@@ -334,10 +387,15 @@ def is_registery_token_valid(secrets_file=user_secrets):
     """Check if register token has expired"""
     registry_token_expires_at = get_docker_expiration_time(secrets_file)
     is_valid = False
+    now = datetime.now().timestamp()
 
     if registry_token_expires_at:
-
-        if registry_token_expires_at > datetime.now().timestamp():
+        if registry_token_expires_at > now:
             is_valid = True
 
+    _logger.debug(
+        f"Registery token: current time: {now}, expiration:"
+        f" {registry_token_expires_at}, valid: {is_valid}"
+    )
+
     return is_valid
