@@ -9,4 +9,4 @@ from embedops_cli.eotools.ci_run import CIRun
 
 if __name__ == "__main__":
     ci_run = CIRun()
-    ci_run.main()
+    ci_run.create_main()
