@@ -8,7 +8,8 @@ Script to send release data to EmbedOps Platform.
 import logging
 import sys
 from os import getenv
-from embedops_cli.api import get_client, Configuration, ReleaseCreateProps
+from embedops_cli.utilities import get_client
+from embedops_cli.api import Configuration, ReleaseCreateProps
 from embedops_cli.api.rest import ApiException
 from embedops_cli.config import settings
 
