@@ -4,10 +4,13 @@
 Utility for telemetry
 * Author(s): Jimmy Gomez
 """
+import logging
 import requests
 from embedops_cli.config import settings
 from .embedops_authorization import get_auth_token
 
+
+_logger = logging.getLogger(__name__)
 measurement_id = settings.get("measurement_id")
 api_secret = settings.get("api_secret")
 CLIENT_ID = "embedops-cli"
@@ -31,4 +34,10 @@ def send_telemetry(event):
     headers = {"Authorization": f"Bearer {token}"}
     host = settings.get("host")
     url = f"{host}/api/v1/users/myself/cli-telemetry"
-    requests.post(url, headers=headers, json=event)
+
+    try:
+        requests.post(url, headers=headers, json=event, timeout=5.0)
+    except requests.ConnectionError as exc:
+        _logger.debug(f"telemetry connection error: {exc}")
+    except requests.Timeout as exc:
+        _logger.debug(f"telemetry timeout error: {exc}")
